@@ -31,7 +31,7 @@ Node = collections.namedtuple('Node', 'dist_own_target_encountered '
                                       'childs')
 
 
-class TreeObsForRailEnv(ObservationBuilder[Node]):
+class TreeObsForRailEnv(ObservationBuilder["RailEnv", Node]):
     """
     TreeObsForRailEnv object.
 
@@ -205,7 +205,7 @@ class TreeObsForRailEnv(ObservationBuilder[Node]):
         else:
             return None
 
-        possible_transitions = self.env.rail.get_transitions(*agent_virtual_position, agent.direction)
+        possible_transitions = self.env.rail.get_transitions((agent_virtual_position, agent.direction))
         num_transitions = fast_count_nonzero(possible_transitions)
 
         # Here information about the agent itself is stored
@@ -319,7 +319,7 @@ class TreeObsForRailEnv(ObservationBuilder[Node]):
                     other_agent_opposite_direction += self.location_has_agent[position]
 
                 # Check number of possible transitions for agent and total number of transitions in cell (type)
-            cell_transitions = self.env.rail.get_transitions(*position, direction)
+            cell_transitions = self.env.rail.get_transitions((position, direction))
             transition_bit = bin(self.env.rail.get_full_transitions(*position))
             total_transitions = transition_bit.count("1")
             crossing_found = False
@@ -459,9 +459,9 @@ class TreeObsForRailEnv(ObservationBuilder[Node]):
         # Start from the current orientation, and see which transitions are available;
         # organize them as [left, forward, right, back], relative to the current orientation
         # Get the possible transitions
-        possible_transitions = self.env.rail.get_transitions(*position, direction)
+        possible_transitions = self.env.rail.get_transitions((position, direction))
         for i, branch_direction in enumerate([(direction + 4 + i) % 4 for i in range(-1, 3)]):
-            if last_is_dead_end and self.env.rail.get_transition((*position, direction),
+            if last_is_dead_end and self.env.rail.get_transition((position, direction),
                                                                  (branch_direction + 2) % 4):
                 # Swap forward and back in case of dead-end, so that an agent can learn that going forward takes
                 # it back
@@ -531,7 +531,7 @@ class TreeObsForRailEnv(ObservationBuilder[Node]):
         return int((direction + 2) % 4)
 
 
-class GlobalObsForRailEnv(ObservationBuilder[Tuple[np.ndarray, np.ndarray, np.ndarray]]):
+class GlobalObsForRailEnv(ObservationBuilder["RailEnv", Tuple[np.ndarray, np.ndarray, np.ndarray]]):
     """
     Gives a global observation of the entire rail environment.
     The observation is composed of the following elements:
@@ -737,3 +737,21 @@ class LocalObsForRailEnv(ObservationBuilder):
             return temp_visible_data
         else:
             return visible, rel_coords
+
+
+class FullEnvObservation(ObservationBuilder["RailEnv", "RailEnv"]):
+    """
+    Returns full env as observation.
+    """
+
+    def __init__(self):
+        pass
+
+    def get(self, handle: AgentHandle = 0) -> "RailEnv":
+        return self.env
+
+    def reset(self):
+        pass
+
+    def set_env(self, env):
+        self.env = env
