@@ -13,9 +13,7 @@ import flatland.envs.timetable_generators as ttg
 from flatland.core.effects_generator import EffectsGenerator, make_multi_effects_generator
 from flatland.core.env import Environment
 from flatland.core.env_observation_builder import ObservationBuilder
-from flatland.core.grid.grid4 import Grid4Transitions
 from flatland.core.grid.grid_utils import Vector2D
-from flatland.core.grid.rail_env_grid import RailEnvTransitionsEnum
 from flatland.envs import agent_chains as ac
 from flatland.envs import line_generators as line_gen
 from flatland.envs import malfunction_generators as mal_gen
@@ -23,11 +21,12 @@ from flatland.envs import persistence
 from flatland.envs import rail_generators as rail_gen
 from flatland.envs.agent_utils import EnvAgent
 from flatland.envs.distance_map import DistanceMap
+from flatland.envs.grid.rail_env_grid import RailEnvTransitionsEnum
 from flatland.envs.malfunction_effects_generators import MalfunctionEffectsGenerator
 from flatland.envs.observations import GlobalObsForRailEnv
 from flatland.envs.rail_env_action import RailEnvActions
 from flatland.envs.rail_grid_transition_map import RailGridTransitionMap
-from flatland.envs.rewards import Rewards
+from flatland.envs.rewards import DefaultRewards, Rewards
 from flatland.envs.step_utils import env_utils
 from flatland.envs.step_utils.state_machine import TrainStateMachine
 from flatland.envs.step_utils.states import TrainState, StateTransitionSignals
@@ -136,7 +135,7 @@ class RailEnv(Environment):
         braking_delta : float
             Determines how much speed is decreased by STOP_MOVING action.
             As speed is between 0.0 and 1.0, braking_delta=-1.0 restores to previous full stop behaviour.
-        rewards : Rewards
+        rewards : DefaultRewards
             The rewards function to use. Defaults to standard settings of Flatland 3 behaviour.
         effects_generator : Optional[EffectsGenerator["RailEnv"]]
             The effects generator that can modify the env at the env of env reset, at the beginning of the env step and at the end of the env step.
@@ -204,7 +203,7 @@ class RailEnv(Environment):
         self.level_free_positions: Set[Vector2D] = set()
 
         if rewards is None:
-            self.rewards = Rewards()
+            self.rewards = DefaultRewards()
         else:
             self.rewards = rewards
 
@@ -388,7 +387,7 @@ class RailEnv(Environment):
 
     def clear_rewards_dict(self):
         """ Reset the rewards dictionary """
-        self.rewards_dict = {i_agent: 0 for i_agent in range(len(self.agents))}
+        self.rewards_dict = {i_agent: self.rewards.empty() for i_agent in range(len(self.agents))}
 
     def get_info_dict(self):
         """
@@ -419,8 +418,9 @@ class RailEnv(Environment):
             ((self._max_episode_steps is not None) and (self._elapsed_steps >= self._max_episode_steps)):
 
             for i_agent, agent in enumerate(self.agents):
-                reward = self.rewards.end_of_episode_reward(agent, self.distance_map, self._elapsed_steps)
-                self.rewards_dict[i_agent] += reward
+                self.rewards_dict[i_agent] = self.rewards.cumulate(
+                    self.rewards_dict[i_agent], self.rewards.end_of_episode_reward(agent, self.distance_map, self._elapsed_steps)
+                )
 
                 self.dones[i_agent] = True
 
@@ -462,10 +462,8 @@ class RailEnv(Environment):
             current_position, current_direction = agent.position, agent.direction
             if current_position is None:  # Agent not added on map yet
                 current_position, current_direction = agent.initial_position, agent.initial_direction
-            _, new_direction_independent, new_position_independent, _, preprocessed_action = self.rail.check_action_on_agent(
-                RailEnvActions.from_value(raw_action),
-                current_position,
-                current_direction
+            _, (new_position_independent, new_direction_independent), _, preprocessed_action = self.rail.check_action_on_agent(
+                RailEnvActions.from_value(raw_action), (current_position, current_direction)
             )
 
             # get desired new_position and new_direction
@@ -510,7 +508,7 @@ class RailEnv(Environment):
                 new_direction = None
 
             if new_position is not None:
-                valid_position_direction = any(self.rail.get_transitions(*new_position, new_direction))
+                valid_position_direction = any(self.rail.get_transitions((new_position, new_direction)))
                 if not valid_position_direction:
                     warnings.warn(f"{(new_position, new_direction)} not valid on the grid."
                                   f" Coming from {(agent.position, agent.direction)} with raw action {raw_action} and preprocessed action {preprocessed_action}. {RailEnvTransitionsEnum(self.rail.get_full_transitions(*agent.position)).name}")
@@ -594,9 +592,18 @@ class RailEnv(Environment):
             have_all_agents_ended &= (agent.state == TrainState.DONE)
 
             ## Update rewards
-            self.rewards_dict[i_agent] += self.rewards.step_reward(agent, agent_transition_data, self.distance_map, self._elapsed_steps)
+            self.rewards_dict[i_agent] = self.rewards.cumulate(
+                self.rewards_dict[i_agent],
+                self.rewards.step_reward(
+                    agent=agent,
+                    agent_transition_data=agent_transition_data,
+                    distance_map=self.distance_map,
+                    elapsed_steps=self._elapsed_steps
+                )
+            )
 
             # update malfunction counter
+            # TODO revise design: updating the malfunction counter after the state transition leaves ugly situation that malfunction_counter == 0 but state is in malfunction - move to begining of step function?
             agent.malfunction_handler.update_counter()
 
             # Off map or on map state and position should match
@@ -684,12 +691,6 @@ class RailEnv(Environment):
         self.obs_dict = self.obs_builder.get_many(list(range(self.get_num_agents())))
         return self.obs_dict
 
-    def get_valid_directions_on_grid(self, row: int, col: int) -> List[int]:
-        """
-        Returns directions in which the agent can move
-        """
-        return Grid4Transitions.get_entry_directions(self.rail.get_full_transitions(row, col))
-
     def _exp_distirbution_synced(self, rate: float) -> float:
         """
         Generates sample from exponential distribution
@@ -794,7 +795,7 @@ class RailEnv(Environment):
                 print("Could Not close window due to:", e)
             self.renderer = None
 
-    def clone_from(self, env: 'RailEnv', obs_builder: Optional[ObservationBuilder[Any]] = None):
+    def clone_from(self, env: 'RailEnv', obs_builder: Optional[ObservationBuilder["RailEnv", Any]] = None):
         from flatland.envs.persistence import RailEnvPersister
         # avoid in-memory references
         env_dict = pickle.loads(pickle.dumps(RailEnvPersister.get_full_state(env)))
