@@ -13,9 +13,9 @@ from flatland.core.grid.grid4 import Grid4TransitionsEnum
 from flatland.core.grid.grid4_utils import direction_to_point, mirror
 from flatland.core.grid.grid_utils import IntVector2DArray, IntVector2D, \
     Vec2dOperations
-from flatland.core.grid.rail_env_grid import RailEnvTransitions, RailEnvTransitionsEnum
 from flatland.core.transition_map import GridTransitionMap
 from flatland.envs import persistence
+from flatland.envs.grid.rail_env_grid import RailEnvTransitions, RailEnvTransitionsEnum
 from flatland.envs.grid4_generators_utils import connect_rail_in_grid_map, connect_straight_line_in_grid_map, \
     fix_inner_nodes, align_cell_to_city
 from flatland.envs.rail_grid_transition_map import RailGridTransitionMap
@@ -127,7 +127,7 @@ def sparse_rail_generator(*args: object, **kwargs: object) -> RailGenerator:
 class SparseRailGen(RailGen):
 
     def __init__(self, max_num_cities: int = 2, grid_mode: bool = False, max_rails_between_cities: int = 2,
-                 max_rail_pairs_in_city: int = 2, seed: int = None, p_level_free: float = 0) -> RailGenerator:
+                 max_rail_pairs_in_city: int = 2, seed: int = None, p_level_free: float = 0, backwards_compatibility_mode: bool = False) -> RailGenerator:
         """
         Generates railway networks with cities and inner city rails
 
@@ -146,6 +146,12 @@ class SparseRailGen(RailGen):
             Initiate the seed
         p_level_free : float
             Percentage of diamond-crossings which are level-free.
+        backwards_compatibility_mode: bool
+            pr 110 Over- and underpasses (aka. level-free diamond crossings) introduced regression in rail_generator.
+            Even if `p_level_free==0`, additional random numbers are drawn for the number of level-free crossings;
+            Setting `p_level_free=0` and `backwards_compatibility_mode=True` restores the previous behaviour for tests depending on the old behaviour.
+            Defaults to `False` (new behaviour).
+
 
         Returns
         -------
@@ -157,6 +163,7 @@ class SparseRailGen(RailGen):
         self.max_rail_pairs_in_city = max_rail_pairs_in_city
         self.seed = seed
         self.p_level_free = p_level_free
+        self.backwards_compatibility_mode = backwards_compatibility_mode
 
     def generate(self, width: int, height: int, num_agents: int, num_resets: int = 0, np_random: RandomState = None) -> RailGeneratorProduct:
         """
@@ -258,13 +265,15 @@ class SparseRailGen(RailGen):
         num_diamond_crossings = np.count_nonzero(grid_map.grid[grid_map.grid == RailEnvTransitionsEnum.diamond_crossing])
         num_level_free_diamond_crossings = math.floor(self.p_level_free * num_diamond_crossings)
         # ceil with probability p_ceil
-        p_ceil = (self.p_level_free * num_diamond_crossings) % 1.0
-        num_level_free_diamond_crossings += np_random.choice([1, 0], p=(p_ceil, 1 - p_ceil))
         level_free_positions = set()
-        if num_level_free_diamond_crossings > 0:
-            choice = np_random.choice(num_diamond_crossings, size=num_level_free_diamond_crossings, replace=False)
-            positions_diamond_crossings = (grid_map.grid == RailEnvTransitionsEnum.diamond_crossing).nonzero()
-            level_free_positions = {tuple(positions_diamond_crossings[choice[i]]) for i in range(len(choice))}
+        if self.p_level_free > 0 or (self.p_level_free == 0 and not self.backwards_compatibility_mode):
+            p_ceil = (self.p_level_free * num_diamond_crossings) % 1.0
+            num_level_free_diamond_crossings += np_random.choice([1, 0], p=(p_ceil, 1 - p_ceil))
+
+            if num_level_free_diamond_crossings > 0:
+                choice = np_random.choice(num_diamond_crossings, size=num_level_free_diamond_crossings, replace=False)
+                positions_diamond_crossings = (grid_map.grid == RailEnvTransitionsEnum.diamond_crossing).nonzero()
+                level_free_positions = {tuple(positions_diamond_crossings[choice[i]]) for i in range(len(choice))}
 
         return grid_map, {
             'agents_hints':
@@ -763,7 +772,7 @@ class SparseRailGen(RailGen):
             # if there are no available transitions, ie (0,0,0,0), then rcPos is invalid
             connected = 0
             for orientation in range(4):
-                connected += grid_transition_map.get_transition((gPos2[0], gPos2[1], orientation), mirror(iDirOut))
+                connected += grid_transition_map.get_transition(((gPos2[0], gPos2[1]), orientation), mirror(iDirOut))
             if connected > 0:
                 incoming_connections[iDirOut] = 1
 
