@@ -1,9 +1,9 @@
 import numpy as np
 from numpy.random import RandomState
 
-from flatland.core.grid.rail_env_grid import RailEnvTransitions
 from flatland.envs.agent_utils import EnvAgent
 from flatland.envs.distance_map import DistanceMap
+from flatland.envs.grid.rail_env_grid import RailEnvTransitions
 from flatland.envs.rail_grid_transition_map import RailGridTransitionMap
 from flatland.envs.rail_trainrun_data_structures import Waypoint
 from flatland.envs.timetable_generators import timetable_generator
@@ -313,6 +313,9 @@ def test_timetable_generator_intermediate():
                  waypoints=[Waypoint(position=(20, 24), direction=0), Waypoint(position=(41, 22), direction=3), Waypoint(position=(39, 8), direction=None)],
                  waypoints_latest_arrival=None, waypoints_earliest_departure=None),
     ]
+    for a in env_agents:
+        # routing flexibility
+        a.waypoints = [[wp] for wp in a.waypoints]
     for a in env_agents:
         print(
             f"EnvAgent(handle={a.handle}, initial_position={a.initial_position}, initial_direction={a.initial_direction}, target={a.target}, direction={a.direction}, waypoints={a.waypoints}, waypoints_latest_arrival=None, waypoints_earliest_departure=None),")
